@@ -1,4 +1,4 @@
-# $Id: Master.pm,v 1.36 2005/12/26 13:57:47 joern Exp $
+# $Id: Master.pm,v 1.40 2006/04/02 14:14:46 joern Exp $
 #-----------------------------------------------------------------------
 # Copyright (C) 2001-2003 Jrn Reder <joern AT zyn.de>.
 # All Rights Reserved. See file COPYRIGHT for details.
@@ -20,6 +20,7 @@ use Video::DVDRip::Cluster::Job;
 use Video::DVDRip::Cluster::Node;
 use Video::DVDRip::Cluster::Project;
 use Video::DVDRip::Cluster::Pipe;
+use Video::DVDRip::Cluster::Scheduler;
 
 use Carp;
 use strict;
@@ -27,32 +28,34 @@ use strict;
 use FileHandle;
 use Data::Dumper;
 
-sub config_filename    { shift->{config_filename} }
-sub data_dir           { shift->{data_dir} }
-sub node_dir           { shift->{node_dir} }
-sub project_dir        { shift->{project_dir} }
-sub nodes              { shift->{nodes} }
-sub projects           { shift->{projects} }
-sub job_id             { shift->{job_id} }
-sub project_id         { shift->{project_id} }
-sub in_job_control     { shift->{in_job_control} }
-sub node_check_watcher { shift->{node_check_watcher} }
-sub rpc_server         { shift->{rpc_server} }
+sub config_filename             { shift->{config_filename}              }
+sub data_dir                    { shift->{data_dir}                     }
+sub node_dir                    { shift->{node_dir}                     }
+sub project_dir                 { shift->{project_dir}                  }
+sub nodes                       { shift->{nodes}                        }
+sub projects                    { shift->{projects}                     }
+sub job_id                      { shift->{job_id}                       }
+sub project_id                  { shift->{project_id}                   }
+sub in_job_control              { shift->{in_job_control}               }
+sub node_check_watcher          { shift->{node_check_watcher}           }
+sub rpc_server                  { shift->{rpc_server}                   }
+sub scheduler                   { shift->{scheduler}                    }
 
-sub set_config_filename    { shift->{config_filename}    = $_[1] }
-sub set_data_dir           { shift->{data_dir}           = $_[1] }
-sub set_node_dir           { shift->{node_dir}           = $_[1] }
-sub set_project_dir        { shift->{project_dir}        = $_[1] }
-sub set_nodes              { shift->{nodes}              = $_[1] }
-sub set_projects           { shift->{projects}           = $_[1] }
-sub set_job_id             { shift->{job_id}             = $_[1] }
-sub set_project_id         { shift->{project_id}         = $_[1] }
-sub set_in_job_control     { shift->{in_job_control}     = $_[1] }
-sub set_node_check_watcher { shift->{node_check_watcher} = $_[1] }
-sub set_rpc_server         { shift->{rpc_server}         = $_[1] }
+sub set_config_filename         { shift->{config_filename}      = $_[1] }
+sub set_data_dir                { shift->{data_dir}             = $_[1] }
+sub set_node_dir                { shift->{node_dir}             = $_[1] }
+sub set_project_dir             { shift->{project_dir}          = $_[1] }
+sub set_nodes                   { shift->{nodes}                = $_[1] }
+sub set_projects                { shift->{projects}             = $_[1] }
+sub set_job_id                  { shift->{job_id}               = $_[1] }
+sub set_project_id              { shift->{project_id}           = $_[1] }
+sub set_in_job_control          { shift->{in_job_control}       = $_[1] }
+sub set_node_check_watcher      { shift->{node_check_watcher}   = $_[1] }
+sub set_rpc_server              { shift->{rpc_server}           = $_[1] }
+sub set_scheduler               { shift->{scheduler}            = $_[1] }
 
 my $MASTER_OBJECT;
-sub get_master {$MASTER_OBJECT}
+sub get_master { $MASTER_OBJECT }
 
 sub new {
     my $class = shift;
@@ -69,6 +72,7 @@ sub new {
         job_id          => 0,
         logger          => $logger,
         rpc_server      => $rpc_server,
+        scheduler       => Video::DVDRip::Cluster::Scheduler->new(),
     }, $class;
 
     $MASTER_OBJECT = $self;
@@ -100,6 +104,19 @@ sub new {
     $self->enable_node_check
         unless $self->node_check_unnecessary;
 
+    foreach my $signal ( "INT", "HUP", "TERM" ) {
+	    Event->signal (
+		    signal => $signal,
+		    cb     => sub {
+                        $self->log(
+                            __x("Got signal {signal}.",
+                                signal => $signal)
+                        );
+                        $self->shutdown("now");
+                    },
+	    );
+    }
+
     return $self;
 }
 
@@ -123,7 +140,6 @@ sub check_prerequisites {
 
 sub node_check_unnecessary {
     my $self = shift;
-    $self->trace_in;
 
     return if $self->rpc_server->get_clients_connected;
     return if @{ $self->job_get_unfinished_projects };
@@ -132,14 +148,13 @@ sub node_check_unnecessary {
 
 sub enable_node_check {
     my $self = shift;
-    $self->trace_in;
 
     return if $self->node_check_watcher;
 
     $self->node_check;
 
     my $watcher = Event->timer(
-        interval => 10,
+        interval => 5,
         cb       => sub { $self->node_check },
         desc     => "node check timer"
     );
@@ -153,7 +168,6 @@ sub enable_node_check {
 
 sub disable_node_check {
     my $self = shift;
-    $self->trace_in;
 
     return if not $self->node_check_watcher;
 
@@ -167,7 +181,6 @@ sub disable_node_check {
 
 sub node_check {
     my $self = shift;
-    $self->trace_in;
 
     my $nodes_list;
     foreach my $node ( @{ $self->nodes } ) {
@@ -218,7 +231,7 @@ sub node_check {
                     if ( not $node->alive and $node->answered_last_ping == 1 )
                     {
                         $self->log(
-                            __x("Node '{node_name}' is still reachable. Will be online in 10 seconds.",
+                            __x("Node '{node_name}' is still reachable. Will be online in 5 seconds.",
                                 node_name => $node_name
                             )
                         );
@@ -227,7 +240,7 @@ sub node_check {
                     if ( not $node->alive and not $node->answered_last_ping )
                     {
                         $self->log(
-                            __x("Node '{node_name}' is now reachable. Will be online in 20 seconds.",
+                            __x("Node '{node_name}' is now reachable. Will be online in 10 seconds.",
                                 node_name => $node_name
                             )
                         );
@@ -283,7 +296,6 @@ sub node_check {
 
 sub hello {
     my $self = shift;
-    $self->trace_in;
 
     $self->enable_node_check;
 
@@ -292,7 +304,6 @@ sub hello {
 
 sub load {
     my $self = shift;
-    $self->trace_in;
 
     my $filename = $self->config_filename;
     if ( not -f $filename ) {
@@ -320,7 +331,6 @@ sub load {
 
 sub save {
     my $self = shift;
-    $self->trace_in;
 
     my $filename = $self->config_filename;
 
@@ -347,7 +357,6 @@ sub save {
 
 sub load_nodes {
     my $self = shift;
-    $self->trace_in;
 
     my $dir = $self->node_dir;
 
@@ -368,7 +377,6 @@ sub load_nodes {
 
 sub load_projects {
     my $self = shift;
-    $self->trace_in;
     my %par = @_;
     my ($project_order) = @par{'project_order'};
 
@@ -382,19 +390,19 @@ sub load_projects {
         );
         my $project = Video::DVDRip::Cluster::Project->new_from_file(
             filename => $filename, );
-        $project->reset_jobs;
-        $project->determine_state;
         push @projects, $project;
     }
 
     $self->set_projects( \@projects );
 
+    $self->scheduler->init;
+
     1;
 }
 
 sub emit_event {
     my $self = shift;
-    my ( $event, @args ) = @_;
+    my ($event, @args) = @_;
 
     my $rpc_server  = $self->rpc_server;
     my $log_clients = $rpc_server->get_logging_clients;
@@ -410,7 +418,6 @@ sub emit_event {
 
 sub add_node {
     my $self = shift;
-    $self->trace_in;
     my %par = @_;
     my ($node) = @par{'node'};
 
@@ -437,7 +444,6 @@ sub add_node {
 
 sub remove_node {
     my $self = shift;
-    $self->trace_in;
     my %par = @_;
     my ($node) = @par{'node'};
 
@@ -460,7 +466,6 @@ sub remove_node {
 
 sub get_project_index {
     my $self = shift;
-    $self->trace_in;
     my %par = @_;
     my ($project) = @par{'project'};
 
@@ -479,7 +484,6 @@ sub get_project_index {
 
 sub project_by_id {
     my $self = shift;
-    $self->trace_in;
     my %par = @_;
     my ($id) = @par{'id'};
 
@@ -493,7 +497,6 @@ sub project_by_id {
 
 sub add_project {
     my $self = shift;
-    $self->trace_in;
     my %par = @_;
     my ($project) = @par{'project'};
 
@@ -526,7 +529,6 @@ sub add_project {
 
 sub move_up_project {
     my $self = shift;
-    $self->trace_in;
     my %par = @_;
     my ($project) = @par{'project'};
 
@@ -549,7 +551,6 @@ sub move_up_project {
 
 sub move_down_project {
     my $self = shift;
-    $self->trace_in;
     my %par = @_;
     my ($project) = @par{'project'};
 
@@ -572,25 +573,24 @@ sub move_down_project {
 
 sub schedule_project {
     my $self = shift;
-    $self->trace_in;
     my %par = @_;
     my ($project) = @par{'project'};
 
     # check for existence
     $self->get_project_index( project => $project );
 
+    # create job plan
+    $project->create_job_plan;
+
     # change project state
     $project->set_state('waiting');
-
-    # change state of all jobs
-    $_->set_state('waiting') foreach @{ $project->jobs };
+    $self->scheduler->add_project($project);
 
     # save project's state
     $project->save;
 
     # emit update events to connected GUI clients
     $self->emit_event("PROJECT_LIST_UPDATE");
-    $self->emit_event( "JOB_PLAN_UPDATE", $project->id );
 
     # maybe the job controller can dispose some work now...
     $self->job_control;
@@ -598,9 +598,39 @@ sub schedule_project {
     1;
 }
 
+sub cancel_project {
+    my $self = shift;
+    my %par = @_;
+    my ($project) = @par{'project'};
+
+    return if $project->state ne 'running';
+    
+    $self->scheduler->cancel_project($project);
+
+    # emit update events to connected GUI clients
+    $self->emit_event("PROJECT_LIST_UPDATE");
+    1;
+}
+
+sub restart_project {
+    my $self = shift;
+    my %par = @_;
+    my ($project) = @par{'project'};
+
+    return if $project->state ne 'cancelled' &&
+              $project->state ne 'error';
+    
+    $self->scheduler->restart_project($project);
+
+    $self->emit_event("PROJECT_LIST_UPDATE");
+
+    $self->scheduler->run;
+
+    1;
+}
+
 sub remove_project {
     my $self = shift;
-    $self->trace_in;
     my %par = @_;
     my ($project) = @par{'project'};
 
@@ -608,17 +638,14 @@ sub remove_project {
     my $i = $self->get_project_index( project => $project );
 
     # check project state
-    return
-        if $project->state  ne 'waiting'
-        and $project->state ne 'not scheduled'
-        and $project->state ne 'merged'
-        and $project->state ne 'audio processed'
-        and $project->state ne 'avi splitted'
-        and $project->state ne 'finished';
+    return if $project->state eq 'running';
 
     unlink $project->filename;
     splice @{ $self->projects }, $i, 1;
 
+    $self->scheduler->remove_project($project)
+        if $project->state ne 'not scheduled';
+
     $self->log(
         __x( "Project {project} removed", project => $project->label ) );
 
@@ -629,7 +656,6 @@ sub remove_project {
 
 sub projects_list {
     my $self = shift;
-    $self->trace_in;
 
     my $nr;
     my @projects;
@@ -646,7 +672,6 @@ sub projects_list {
 
 sub jobs_list {
     my $self = shift;
-    $self->trace_in;
     my %par = @_;
     my ($project_id) = @par{'project_id'};
 
@@ -657,7 +682,6 @@ sub jobs_list {
 
 sub nodes_list {
     my $self = shift;
-    $self->trace_in;
 
     my $nr;
     my @nodes;
@@ -667,9 +691,9 @@ sub nodes_list {
             $node->name,
             ++$nr,
             $node->name,
-            ( $node->assigned_job ? $node->assigned_job->node_label : undef ),
+            ( $node->assigned_job ? $node->assigned_job->get_info : undef ),
             (     $node->assigned_job
-                ? $node->assigned_job->progress
+                ? $node->assigned_job->get_progress_stats
                 : $node->state
             ),
             ];
@@ -680,172 +704,14 @@ sub nodes_list {
 
 sub job_control {
     my $self = shift;
-    $self->trace_in;
-    return if $self->in_job_control;
 
-    $self->set_in_job_control(1);
-
-    # 1. move waiting projects before not scheduled projects
-    $self->job_sort_projects;
-
-    # 2. delegate pending jobs
-    $self->job_delegate_pending_jobs;
-
-    $self->set_in_job_control(0);
+    $self->scheduler->run;
 
     1;
 }
 
-sub job_sort_projects {
-    my $self = shift;
-    $self->trace_in;
-
-    # divide projects in order to not scheduled and scheduled projects
-    my @not_scheduled;
-    my @scheduled;
-    my @finished;
-    foreach my $project ( @{ $self->projects } ) {
-        if ( $project->state eq 'not scheduled' ) {
-            push @not_scheduled, $project;
-        }
-        elsif ( $project->state eq 'finished' ) {
-            push @finished, $project;
-        }
-        else {
-            push @scheduled, $project;
-        }
-    }
-
-    # define project list with scheduled first, then the not
-    # scheduled projects
-    my @projects = ( @scheduled, @not_scheduled, @finished );
-
-    $self->set_projects( \@projects );
-
-    1;
-}
-
-sub job_delegate_pending_jobs {
-    my $self = shift;
-    $self->trace_in;
-
-    # get idle nodes
-    my $nodes = $self->job_get_idle_nodes;
-
-    # nothing to do if no node is idle
-    return 1 if not @{$nodes};
-
-    # get waiting projects
-    my $projects = $self->job_get_unfinished_projects;
-
-    # nothing to do if no project is in the queue for processing
-    return 1 if not @{$projects};
-
-    # in this array we remember local jobs which
-    # are skipped due to a lack of local nodes
-    my @skipped_local_access_jobs;
-
-    # now check for jobs for each project, in order of priority
-    foreach my $project ( @{$projects} ) {
-
-        # get out here if no idles nodes are left
-        last if not @{$nodes};
-
-        # create list of jobs, which are waiting, with local_access
-        # jobs first (because we actively search local nodes later,
-        # this prevents blocking a local node with a non local job,
-        # just a round before we try to schedule a local job).
-        my ( @local_jobs, @other_jobs );
-
-        foreach my $job ( @{ $project->jobs } ) {
-
-            # skip jobs that are finished or running
-            next
-                if $job->state eq 'finished'
-                or $job->state eq 'running'
-                or $job->state eq 'aborted';
-
-            # We do not check the dependency state of
-            # the job here, because this is expensive.
-            # This is done later in the scheduler loop,
-            # because it exits if no idle nodes are left,
-            # which is regularly very early, so computing
-            # the dependecies here in advance makes no
-            # sense at all.
-
-            if ( $job->prefer_local_access ) {
-                push @local_jobs, $job;
-            }
-            else {
-                push @other_jobs, $job;
-            }
-        }
-
-        # check if we can start jobs
-        foreach my $job ( @local_jobs, @other_jobs ) {
-            last if not @{$nodes};
-            next if not $job->dependency_ok;
-
-            if ( $job->prefer_local_access ) {
-
-                # search a node with local access
-                my $i = 0;
-                foreach my $node ( @{$nodes} ) {
-                    last if $node->data_is_local;
-                    ++$i;
-                }
-                if ( $i < @{$nodes} ) {
-
-                    # found one: start the job on it
-                    $job->start_job( node => splice( @{$nodes}, $i, 1 ) );
-                    next;
-
-                }
-                else {
-
-                    # no local node found. skip this job.
-                    # schedule it later if we have idle
-                    # nodes left after this round.
-                    push @skipped_local_access_jobs, $job;
-                    next;
-                }
-
-            }
-
-            # this is a normal job: start it on the next idle node
-            $job->start_job( node => shift @{$nodes} );
-        }
-    }
-
-    # do we have skipped local jobs and have idle nodes left?
-    if ( @{$nodes} and @skipped_local_access_jobs ) {
-        foreach my $job (@skipped_local_access_jobs) {
-            $job->start_job( node => shift @{$nodes} );
-            last if not @{$nodes};
-        }
-    }
-
-    1;
-}
-
-sub job_get_idle_nodes {
-    my $self = shift;
-    $self->trace_in;
-
-    # create a list of nodes which have the 'idle' state
-    my @idle_nodes;
-    foreach my $node ( @{ $self->nodes } ) {
-        push @idle_nodes, $node if $node->state eq 'idle';
-    }
-
-    @idle_nodes = sort { $b->speed <=> $a->speed } @idle_nodes;
-
-    return \@idle_nodes;
-}
-
 sub job_get_unfinished_projects {
     my $self = shift;
-    $self->trace_in;
 
     my @projects;
 
@@ -862,7 +728,19 @@ sub job_get_unfinished_projects {
 
 sub shutdown {
     my $self = shift;
-    $self->trace_in;
+    my ($now) = @_;
+
+    my $exec_flow_group = $self->scheduler->get_exec_flow_group;
+    
+    if ( $exec_flow_group->get_state eq 'running' ) {
+        $self->log( __"Master shutdown. Stopping active jobs." );
+        $exec_flow_group->cancel;
+    }
+    else {
+        $self->log( __"Master shutdown" );
+        Event::unloop_all();
+        return;
+    }
 
     Event->timer(
         interval => 2,
@@ -876,19 +754,8 @@ sub shutdown {
 
 }
 
-sub get_next_job_id {
-    my $self = shift;
-    $self->trace_in;
-
-    my $job_id = $self->set_job_id( 1 + $self->job_id );
-    $self->save;
-
-    return $job_id;
-}
-
 sub get_online_nodes_cnt {
     my $self = shift;
-    $self->trace_in;
 
     my $cnt = 0;
     foreach my $node ( @{ $self->nodes } ) {
@@ -953,4 +820,16 @@ sub node_test {
     1;
 }
 
+sub exec_flow_job {
+    my $self = shift;
+    
+    return $self->scheduler->get_exec_flow_group;
+}
+
+sub get_job_from_id {
+    my $self = shift;
+    my ($job_id) = @_;
+    return $self->scheduler->get_jobs_by_id->{$job_id};
+}
+
 1;
