@@ -1,4 +1,4 @@
-# $Id: Node.pm,v 1.35 2006/01/02 19:59:36 joern Exp $
+# $Id: Node.pm,v 1.39 2006/04/02 21:07:35 joern Exp $
 
 #-----------------------------------------------------------------------
 # Copyright (C) 2001-2003 Jrn Reder <joern AT zyn.de>.
@@ -20,25 +20,26 @@ use strict;
 use FileHandle;
 use Data::Dumper;
 
-sub state              { shift->{state} }
-sub filename           { shift->{filename} }
-sub name               { shift->{name} }
-sub hostname           { my $s = shift; $s->{hostname} || $s->{name} }
-sub data_base_dir      { shift->{data_base_dir} }
-sub is_master          { shift->{is_master} }
-sub data_is_local      { shift->{data_is_local} }
-sub username           { shift->{username} }
-sub ssh_cmd            { shift->{ssh_cmd} }
-sub speed              { shift->{speed} }
-sub tc_options         { shift->{tc_options} }
-sub answered_last_ping { shift->{answered_last_ping} }
+sub state                       { shift->{state}                        }
+sub filename                    { shift->{filename}                     }
+sub name                        { shift->{name}                         }
+sub hostname                    { my $s = shift; $s->{hostname} || $s->{name} }
+sub data_base_dir               { shift->{data_base_dir}                }
+sub is_master                   { shift->{is_master}                    }
+sub data_is_local               { shift->{data_is_local}                }
+sub username                    { shift->{username}                     }
+sub ssh_cmd                     { shift->{ssh_cmd}                      }
+sub speed                       { shift->{speed}                        }
+sub tc_options                  { shift->{tc_options}                   }
+sub answered_last_ping          { shift->{answered_last_ping}           }
+sub speed_index                 { shift->{speed_index}                  }
 
-sub progress_cnt        { shift->{progress_cnt} }
-sub progress_max        { shift->{progress_max} }
-sub progress_merge      { shift->{progress_merge} }
-sub progress_start_time { shift->{progress_start_time} }
-sub assigned_job        { shift->{assigned_job} }
-sub assigned_chunk      { shift->{assigned_chunk} }
+sub progress_cnt                { shift->{progress_cnt}                 }
+sub progress_max                { shift->{progress_max}                 }
+sub progress_merge              { shift->{progress_merge}               }
+sub progress_start_time         { shift->{progress_start_time}          }
+sub assigned_job                { shift->{assigned_job}                 }
+sub assigned_chunk              { shift->{assigned_chunk}               }
 
 sub set_state {
     my $self = shift, my ($state) = @_;
@@ -51,30 +52,31 @@ sub set_state {
     return $state;
 }
 
-sub set_filename           { shift->{filename}           = $_[1] }
-sub set_name               { shift->{name}               = $_[1] }
-sub set_hostname           { shift->{hostname}           = $_[1] }
-sub set_data_base_dir      { shift->{data_base_dir}      = $_[1] }
-sub set_is_master          { shift->{is_master}          = $_[1] }
-sub set_data_is_local      { shift->{data_is_local}      = $_[1] }
-sub set_username           { shift->{username}           = $_[1] }
-sub set_ssh_cmd            { shift->{ssh_cmd}            = $_[1] }
-sub set_speed              { shift->{speed}              = $_[1] }
-sub set_tc_options         { shift->{tc_options}         = $_[1] }
-sub set_answered_last_ping { shift->{answered_last_ping} = $_[1] }
+sub set_filename                { shift->{filename}             = $_[1] }
+sub set_name                    { shift->{name}                 = $_[1] }
+sub set_hostname                { shift->{hostname}             = $_[1] }
+sub set_data_base_dir           { shift->{data_base_dir}        = $_[1] }
+sub set_is_master               { shift->{is_master}            = $_[1] }
+sub set_data_is_local           { shift->{data_is_local}        = $_[1] }
+sub set_username                { shift->{username}             = $_[1] }
+sub set_ssh_cmd                 { shift->{ssh_cmd}              = $_[1] }
+sub set_speed                   { shift->{speed}                = $_[1] }
+sub set_tc_options              { shift->{tc_options}           = $_[1] }
+sub set_answered_last_ping      { shift->{answered_last_ping}   = $_[1] }
+sub set_speed_index             { shift->{speed_index}          = $_[1] }
 
-sub set_progress_cnt        { shift->{progress_cnt}        = $_[1] }
-sub set_progress_max        { shift->{progress_max}        = $_[1] }
-sub set_progress_merge      { shift->{progress_merge}      = $_[1] }
-sub set_progress_start_time { shift->{progress_start_time} = $_[1] }
-sub set_assigned_job        { shift->{assigned_job}        = $_[1] }
-sub set_assigned_chunk      { shift->{assigned_chunk}      = $_[1] }
+sub set_progress_cnt            { shift->{progress_cnt}         = $_[1] }
+sub set_progress_max            { shift->{progress_max}         = $_[1] }
+sub set_progress_merge          { shift->{progress_merge}       = $_[1] }
+sub set_progress_start_time     { shift->{progress_start_time}  = $_[1] }
+sub set_assigned_job            { shift->{assigned_job}         = $_[1] }
+sub set_assigned_chunk          { shift->{assigned_chunk}       = $_[1] }
 
-sub test_finished { shift->{test_finished} }
-sub test_result   { shift->{test_result} }
+sub test_finished               { shift->{test_finished}                }
+sub test_result                 { shift->{test_result}                  }
 
-sub set_test_finished { shift->{test_finished} = $_[1] }
-sub set_test_result   { shift->{test_result}   = $_[1] }
+sub set_test_finished           { shift->{test_finished}        = $_[1] }
+sub set_test_result             { shift->{test_result}          = $_[1] }
 
 sub alive { shift->{alive} }
 
@@ -157,7 +159,6 @@ sub new_from_file {
 
 sub load {
     my $self = shift;
-    $self->trace_in;
 
     my $filename = $self->filename;
     croak "no filename set"      if not $filename;
@@ -179,7 +180,6 @@ sub load {
 
 sub save {
     my $self = shift;
-    $self->trace_in;
 
     my $filename = $self->filename;
     confess "not filename set" if not $filename;
@@ -208,14 +208,31 @@ sub save {
     1;
 }
 
-sub get_popen_code {
-    my $self      = shift;
-    my %par       = @_;
-    my ($command) = @par{'command'};
+sub prepare_command {
+    my $self = shift;
+    my ($command, $job) = @_;
+
+    if ( $job ) {
+        my %params = (
+            DVDRIP_NODE_DATA_BASE_DIR   => $self->data_base_dir,
+            DVDRIP_NODE_NAME            => $self->name,
+            DVDRIP_JOB_PSU              => sprintf("%02d",$job->get_stash->{psu}),
+            DVDRIP_JOB_CHUNK            => sprintf("%05d",$job->get_stash->{chunk}),
+            DVDRIP_JOB_AVI_NR           => sprintf("%02d",$job->get_stash->{avi_nr}),
+            DVDRIP_JOB_CHUNK_CNT        => $job->get_stash->{chunk_cnt},
+        );
+        $command =~ s/(DVDRIP_[A-Z_]+)/$params{$1}/eg;
+    }
+
+    if ( $self->is_master ) {
+        $command = "execflow $command" unless $command =~ /execflow/;
+        $command = "umask 0002; $command";
+        return $command;
+    }
 
     $command = "umask 0002; $command";
 
-    return $command if $self->is_master;
+    warn "Node's transcode options currently ignored" if $self->tc_options;
 
     #-- Set LD_ASSUME_KERNEL on the nodes as well,
     #-- if set on the master.
@@ -228,9 +245,9 @@ sub get_popen_code {
 
     $ssh_cmd .= " -o PreferredAuthentications=publickey";
 
-    $command =~ s/dvdrip-exec(?:\s*-n\s*\d+\s*)?//g;
+    $command =~ s/execflow(?:\s*-n\s*\d+\s*)?//g;
     $command =~ s/"/\\"/g;
-    $command = qq{dvdrip-exec $ssh_cmd $username\@$name "$command"};
+    $command = qq{execflow $ssh_cmd $username\@$name "$command"};
 
     return $command;
 }
@@ -254,7 +271,7 @@ sub reset {
 sub progress {
     my $self = shift;
     return "" if not $self->assigned_job;
-    return $self->assigned_job->progress;
+    return $self->assigned_job->get_progress_text;
 }
 
 sub stop {
@@ -318,7 +335,7 @@ sub run_tests {
     # get test command for this node
     my $command = $self->get_test_command;
 
-    my $popen_command = $self->get_popen_code( command => "($command) 2>&1" );
+    my $popen_command = $self->prepare_command($command);
 
     my $output = "";
     Video::DVDRip::Cluster::Pipe->new(
